@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -10,9 +10,9 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -23,8 +23,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
@@ -653,9 +653,10 @@ bool QQmlImportNamespace::Import::resolveType(QQmlTypeLoader *typeLoader,
         }
 
         if (candidate != end) {
+            QQmlType *returnType = getTypeForUrl(componentUrl, type, isCompositeSingleton, 0);
             if (type_return)
-                *type_return = getTypeForUrl(componentUrl, type, isCompositeSingleton, 0);
-            return (*type_return != 0);
+                *type_return = returnType;
+            return returnType != 0;
         }
     } else if (!isLibrary) {
         QString qmlUrl;
@@ -667,15 +668,7 @@ bool QQmlImportNamespace::Import::resolveType(QQmlTypeLoader *typeLoader,
         };
         for (uint i = 0; i < sizeof(urlsToTry) / sizeof(urlsToTry[0]); ++i) {
             const QString url = urlsToTry[i];
-
-            if (QQmlFile::isBundle(url)) {
-                exists = QQmlFile::bundleFileExists(url, typeLoader->engine());
-            } else {
-                exists = !typeLoader->absoluteFilePath(QQmlFile::urlToLocalFileOrQrc(url)).isEmpty();
-                if (!exists)
-                    exists = QQmlMetaType::findCachedCompilationUnit(QUrl(url));
-            }
-
+            exists = !typeLoader->absoluteFilePath(QQmlFile::urlToLocalFileOrQrc(url)).isEmpty();
             if (exists) {
                 qmlUrl = url;
                 break;
@@ -687,9 +680,10 @@ bool QQmlImportNamespace::Import::resolveType(QQmlTypeLoader *typeLoader,
                 if (typeRecursionDetected)
                     *typeRecursionDetected = true;
             } else {
+                QQmlType *returnType = getTypeForUrl(qmlUrl, type, false, 0);
                 if (type_return)
-                    *type_return = getTypeForUrl(qmlUrl, type, false, 0);
-                return (*type_return) != 0;
+                    *type_return = returnType;
+                return returnType != 0;
             }
         }
     }
@@ -879,7 +873,7 @@ bool QQmlImportsPrivate::populatePluginPairVector(QVector<StaticPluginPair> &res
 /*!
 Import an extension defined by a qmldir file.
 
-\a qmldirFilePath is either a raw file path, or a bundle url.
+\a qmldirFilePath is a raw file path.
 */
 bool QQmlImportsPrivate::importExtension(const QString &qmldirFilePath,
                                          const QString &uri,
@@ -930,7 +924,7 @@ bool QQmlImportsPrivate::importExtension(const QString &qmldirFilePath,
             QString resolvedFilePath = database->resolvePlugin(typeLoader, qmldirPath, plugin.path, plugin.name);
             if (!resolvedFilePath.isEmpty()) {
                 dynamicPluginsFound++;
-                if (!database->importDynamicPlugin(resolvedFilePath, uri, typeNamespace, errors)) {
+                if (!database->importDynamicPlugin(resolvedFilePath, uri, typeNamespace, vmaj, errors)) {
                     if (errors) {
                         // XXX TODO: should we leave the import plugin error alone?
                         // Here, we pop it off the top and coalesce it into this error's message.
@@ -966,7 +960,7 @@ bool QQmlImportsPrivate::importExtension(const QString &qmldirFilePath,
                         if (versionUri == metaTagUri.toString()) {
                             staticPluginsFound++;
                             QObject *instance = pair.first.instance();
-                            if (!database->importStaticPlugin(instance, basePath, uri, typeNamespace, errors)) {
+                            if (!database->importStaticPlugin(instance, basePath, uri, typeNamespace, vmaj, errors)) {
                                 if (errors) {
                                     QQmlError poppedError = errors->takeFirst();
                                     QQmlError error;
@@ -1012,17 +1006,11 @@ bool QQmlImportsPrivate::getQmldirContent(const QString &qmldirIdentifier, const
     Q_ASSERT(errors);
     Q_ASSERT(qmldir);
 
-    *qmldir = typeLoader->qmldirContent(qmldirIdentifier, uri);
+    *qmldir = typeLoader->qmldirContent(qmldirIdentifier);
     if (*qmldir) {
         // Ensure that parsing was successful
         if ((*qmldir)->hasError()) {
-            QUrl url;
-
-            if (QQmlFile::isBundle(qmldirIdentifier))
-                url = QUrl(qmldirIdentifier);
-            else
-                url = QUrl::fromLocalFile(qmldirIdentifier);
-
+            QUrl url = QUrl::fromLocalFile(qmldirIdentifier);
             const QList<QQmlError> qmldirErrors = (*qmldir)->errors(uri);
             for (int i = 0; i < qmldirErrors.size(); ++i) {
                 QQmlError error = qmldirErrors.at(i);
@@ -1328,30 +1316,7 @@ bool QQmlImportsPrivate::addFileImport(const QString& uri, const QString &prefix
 
     QString qmldirIdentifier;
 
-    if (QQmlFile::isBundle(qmldirUrl)) {
-
-        QString dir = resolveLocalUrl(base, importUri);
-        Q_ASSERT(QQmlFile::isBundle(dir));
-        if (!QQmlFile::bundleDirectoryExists(dir, typeLoader->engine())) {
-            if (!isImplicitImport) {
-                QQmlError error;
-                error.setDescription(QQmlImportDatabase::tr("\"%1\": no such directory").arg(uri));
-                error.setUrl(QUrl(qmldirUrl));
-                errors->prepend(error);
-            }
-            return false;
-        }
-
-        // Transforms the (possible relative) uri into our best guess relative to the
-        // import paths.
-        importUri = resolvedUri(dir, database);
-        if (importUri.endsWith(Slash))
-            importUri.chop(1);
-
-        if (QQmlFile::bundleFileExists(qmldirUrl, typeLoader->engine()))
-            qmldirIdentifier = qmldirUrl;
-
-    } else if (QQmlFile::isLocalFile(qmldirUrl)) {
+    if (QQmlFile::isLocalFile(qmldirUrl)) {
 
         QString localFileOrQrc = QQmlFile::urlToLocalFileOrQrc(qmldirUrl);
         Q_ASSERT(!localFileOrQrc.isEmpty());
@@ -1555,12 +1520,12 @@ bool QQmlImports::locateQmldir(QQmlImportDatabase *importDb,
 
 bool QQmlImports::isLocal(const QString &url)
 {
-    return QQmlFile::isBundle(url) || !QQmlFile::urlToLocalFileOrQrc(url).isEmpty();
+    return !QQmlFile::urlToLocalFileOrQrc(url).isEmpty();
 }
 
 bool QQmlImports::isLocal(const QUrl &url)
 {
-    return QQmlFile::isBundle(url) || !QQmlFile::urlToLocalFileOrQrc(url).isEmpty();
+    return !QQmlFile::urlToLocalFileOrQrc(url).isEmpty();
 }
 
 void QQmlImports::setDesignerSupportRequired(bool b)
@@ -1676,7 +1641,7 @@ QString QQmlImportDatabase::resolvePlugin(QQmlTypeLoader *typeLoader,
   \row \li Unix/Linux  \li \c .so
   \row \li AIX  \li \c .a
   \row \li HP-UX       \li \c .sl, \c .so (HP-UXi)
-  \row \li Mac OS X    \li \c .dylib, \c .bundle, \c .so
+  \row \li OS X    \li \c .dylib, \c .bundle, \c .so
   \endtable
 
   Version number on unix are ignored.
@@ -1849,7 +1814,7 @@ void QQmlImportDatabase::setImportPathList(const QStringList &paths)
     \internal
 */
 bool QQmlImportDatabase::registerPluginTypes(QObject *instance, const QString &basePath,
-                                      const QString &uri, const QString &typeNamespace, QList<QQmlError> *errors)
+                                      const QString &uri, const QString &typeNamespace, int vmaj, QList<QQmlError> *errors)
 {
     if (qmlImportTrace())
         qDebug().nospace() << "QQmlImportDatabase::registerPluginTypes: " << uri << " from " << basePath;
@@ -1871,7 +1836,7 @@ bool QQmlImportDatabase::registerPluginTypes(QObject *instance, const QString &b
     {
         // Create a scope for QWriteLocker to keep it as narrow as possible, and
         // to ensure that we release it before the call to initalizeEngine below
-        QWriteLocker lock(QQmlMetaType::typeRegistrationLock());
+        QMutexLocker lock(QQmlMetaType::typeRegistrationLock());
 
         if (!typeNamespace.isEmpty()) {
             // This is an 'identified' module
@@ -1885,7 +1850,7 @@ bool QQmlImportDatabase::registerPluginTypes(QObject *instance, const QString &b
                 return false;
             }
 
-            if (QQmlMetaType::namespaceContainsRegistrations(typeNamespace)) {
+            if (QQmlMetaType::namespaceContainsRegistrations(typeNamespace, vmaj)) {
                 // Other modules have already installed to this namespace
                 if (errors) {
                     QQmlError error;
@@ -1932,7 +1897,7 @@ bool QQmlImportDatabase::registerPluginTypes(QObject *instance, const QString &b
     \internal
 */
 bool QQmlImportDatabase::importStaticPlugin(QObject *instance, const QString &basePath,
-                                      const QString &uri, const QString &typeNamespace, QList<QQmlError> *errors)
+                                      const QString &uri, const QString &typeNamespace, int vmaj, QList<QQmlError> *errors)
 {
 #ifndef QT_NO_LIBRARY
     // Dynamic plugins are differentiated by their filepath. For static plugins we
@@ -1956,7 +1921,7 @@ bool QQmlImportDatabase::importStaticPlugin(QObject *instance, const QString &ba
         plugin.loader = 0;
         plugins->insert(uniquePluginID, plugin);
 
-        if (!registerPluginTypes(instance, basePath, uri, typeNamespace, errors))
+        if (!registerPluginTypes(instance, basePath, uri, typeNamespace, vmaj, errors))
             return false;
     }
 
@@ -1979,7 +1944,7 @@ bool QQmlImportDatabase::importStaticPlugin(QObject *instance, const QString &ba
     \internal
 */
 bool QQmlImportDatabase::importDynamicPlugin(const QString &filePath, const QString &uri,
-                                             const QString &typeNamespace, QList<QQmlError> *errors)
+                                             const QString &typeNamespace, int vmaj, QList<QQmlError> *errors)
 {
 #ifndef QT_NO_LIBRARY
     QFileInfo fileInfo(filePath);
@@ -2015,9 +1980,8 @@ bool QQmlImportDatabase::importDynamicPlugin(const QString &filePath, const QStr
                     QQmlError error;
                     error.setDescription(loader->errorString());
                     errors->prepend(error);
-
-                    delete loader;
                 }
+                delete loader;
                 return false;
             }
         } else {
@@ -2033,7 +1997,7 @@ bool QQmlImportDatabase::importDynamicPlugin(const QString &filePath, const QStr
             plugins->insert(absoluteFilePath, plugin);
 
             // Continue with shared code path for dynamic and static plugins:
-            if (!registerPluginTypes(instance, fileInfo.absolutePath(), uri, typeNamespace, errors))
+            if (!registerPluginTypes(instance, fileInfo.absolutePath(), uri, typeNamespace, vmaj, errors))
                 return false;
         }
 
